// TODO: make error handling better (specially on all option)
part of '../../../git_assistant.dart';

/// Git push command class
///
/// Provides push command functionalities
///
/// ```dart
/// var git = Git();
/// // 'git push origin master'
/// print(git.push.to(remote: 'origin', refspec: 'master'));
/// git.push.allOpt().exec(); // Runs 'git push --all'
/// ```
class PushGitChain extends BaseGitChain {
  PushGitChain(String chainName, BaseGitChain prevChain)
      : super(chainName, prevChain);

  PushGitChain _createPushOption(String chainName) {
    return PushGitChain(chainName, this);
  }

  /// Generates --force option
  PushGitChain forceOpt() {
    return _createPushOption('--force');
  }

  /// Generates --quiet option
  PushGitChain quietOpt() {
    return _createPushOption('--quiet');
  }

  /// Generates --all option
  ///
  /// As you know you can't use --all and 'remote branch' together. In the `git-assistant`
  /// package you shouldn't use allOpt() and to(remote: '', refspec: '') together.
  ///
  /// ```dart
  /// var git = Git();
  /// git.push.allOpt().exec(); // it's fine
  /// // you will get exception from git itself
  /// git.push.allOpt().to(remote: 'origin', refspec: 'master').exec();
  /// ```
  PushGitChain allOpt() {
    return _createPushOption('--all');
  }

  /// Generates --dry-run option
  PushGitChain dryRunOpt() {
    return _createPushOption('--dry-run');
  }

  /// Generates --set-upstream option.
  ///
  /// Adds 'remote refspec' if both **remote** and **refspec** are provided.
  PushGitChain setUpstreamOpt({String? remote, String? refspec}) {
    if (remote != null && refspec != null) {
      return _createPushOption('--set-upstream $remote $refspec');
    }
    return _createPushOption('--set-upstream');
  }

  /// Adds 'remote refspec' end of the main chain both them are required.
  PushGitChain to({required String remote, required String refspec}) {
    return _createPushOption('$remote $refspec');
  }
}
