import 'dart:io';

import 'package:git_assistant/git_assistant.dart';
import 'package:test/test.dart';

void main() {
  group('PushGitChain class', () {
    var git = Git();

    test('test push command generator', () {
      expect(
        git.push.allOpt().quietOpt().toString(),
        'git push --all --quiet',
      );
      expect(
        git.push.forceOpt().allOpt().toString(),
        'git push --force --all',
      );
      expect(
        git.push
            .setUpstreamOpt(remote: 'origin', refspec: 'master:qa/dev')
            .toString(),
        'git push --set-upstream origin master:qa/dev',
      );
      expect(
        git.push
            .to(remote: 'origin', refspec: 'feature/new-feature')
            .toString(),
        'git push origin feature/new-feature',
      );
      expect(
        git.push.setUpstreamOpt(remote: 'remote').toString(),
        'git push --set-upstream',
      );
      expect(
        git.push
            .dryRunOpt()
            .to(remote: 'origin', refspec: 'test/data')
            .toString(),
        'git push --dry-run origin test/data',
      );
      expect(
        // ignore: invalid_use_of_protected_member
        git.push.setUpstreamOpt(remote: 'remote', refspec: 'dev').argList,
        ['git', 'push', '--set-upstream remote dev'],
      );
      expect(
        // ignore: invalid_use_of_protected_member
        git.push.to(remote: 'origin', refspec: 'master:feature/remote').argList,
        ['git', 'push', 'origin master:feature/remote'],
      );
    });

    test('test push command executor', () {
      // because we don't wanna push the changes to the remote in real testing
      // we used dry run mode.
      expect(
        git.push.dryRunOpt().to(remote: 'origin', refspec: 'master').exec(),
        Process.runSync('git', ['push', '--dry-run', 'origin master']).stdout,
      );
      expect(
        git.push
            .quietOpt()
            .dryRunOpt()
            .to(remote: 'origin', refspec: 'master')
            .exec(),
        Process.runSync(
          'git',
          ['push', '--quiet', '--dry-run', 'origin master'],
        ).stdout,
      );
    });
  });
}
